@@ -94,6 +94,7 @@ typedef enum
         GSM_MANAGER_ERROR_ALREADY_REGISTERED,
         GSM_MANAGER_ERROR_NOT_REGISTERED,
         GSM_MANAGER_ERROR_INVALID_OPTION,
+        GSM_MANAGER_ERROR_LOCKED_DOWN,
         GSM_MANAGER_NUM_ERRORS
 } GsmManagerError;
 
@@ -120,6 +121,9 @@ gboolean            gsm_manager_get_failsafe                   (GsmManager     *
 gboolean            gsm_manager_add_autostart_app              (GsmManager     *manager,
                                                                 const char     *path,
                                                                 const char     *provides);
+gboolean            gsm_manager_add_required_app               (GsmManager     *manager,
+                                                                const char     *path,
+                                                                const char     *provides);
 gboolean            gsm_manager_add_autostart_apps_from_dir    (GsmManager     *manager,
                                                                 const char     *path);
 gboolean            gsm_manager_add_legacy_session_apps        (GsmManager     *manager,
